import sys.io.File;
import highlighter.Highlighter; //https://lib.haxe.org/p/html-haxe-code-highlighter/
using StringTools;

class SiteGen {
	/** 
	 * cf plugin SEO Quake pour Firefox
	 * meta descritpion trop courte
	 * h1, h2 ... générés par Markdown
	 * une image sans alt
	 * robots.txt, sitemap.xml, favicon
	 * Microformat, Schema.org, canonical
	 * Menu HTML/CSS seuls : https://medialoot.com/blog/how-to-create-a-responsive-navigation-menu-using-only-css/
	 * use haxe.Template
	 */

	public static function main() {
		doIndex();
		doNews();
		doUML();
		doExamples();
	}

	static function doIndex() {
		var html = header("Home");
		html += mdToW3Html(Xml.parse(embed("h1", "article", highlight(Markdown.markdownToHtml(File.getContent("README.md"))))));
		html += footer();
		File.saveContent("doc/web/index.html", html);
	}

	static function doNews() {
		var html = header("News");
		html += mdToW3Html(Xml.parse(embed("h1", "article", Markdown.markdownToHtml(File.getContent("doc/ChangeLog.md")))));
		html += footer();
		File.saveContent("doc/web/news.html", html);
	}

	static function doUML() {
		var html = header("UML");
		var mdContent = 'Containers\n==========\n\n' +
						'![UML container classes](https://haxe.tuxfamily.org/buw/png/uml1-containers.png "UML container classes")\n\n\n' +
						'Base widgets\n============\n\n' +
						'![UML base widgets](https://haxe.tuxfamily.org/buw/png/uml2-base-widgets.png "UML base widgets")\n\n\n' +
						'Advanced controls\n=================\n\n' +
						'![UML advanced controls](https://haxe.tuxfamily.org/buw/png/uml3-advanced-controls.png "UML advanced controls")\n';
		html += mdToW3Html(Xml.parse(embed("h1", "article", Markdown.markdownToHtml(mdContent))));
		html += footer();
		File.saveContent("doc/web/uml.html", html);
	}

	static function doExamples() {
		var attr = ~/class=([a-z]+)/g; //XHTML fix
		var html = header("Examples");
		var section = "";
		for (i in [ "1", "2", "3a", "3b", "4", "5", "6" ]) { //sys.FileSystem.readDirectory(dir : String) : Array<String>
			section += '<article id="example$i">\n';
			section += '<h1 class="w3-theme">Example $i</h1>\n';
			section += '<h2 class="w3-text-theme">Screenshot</h2>\n<p><img src="png/example$i.png" alt="Example $i screenshot"/></p>\n';
			section += '<h2>Source code</h2>\n<pre class="w3-theme-light"><code class="prettyprint">' +
				attr.replace(Highlighter.syntaxHighlightHaxe(File.getContent('samples/Source/Example$i.hx')), "class=\"$1\"") + "</code></pre>\n";
			section += "</article>";
		}
		html += section;
		html += footer();
		File.saveContent("doc/web/examples.html", html);
	}

	static function mdToW3Html(xml : Xml) : String {
		for (article in xml.elementsNamed("article")) {
			article.set("id", article.firstChild().firstChild().nodeValue.toLowerCase()); //article->h1
			article.firstChild().set("class", "w3-theme");
			for (tag in [ "h2", "p", "ul" ]) {
				for (t in article.elementsNamed(tag)) {
					t.set("class", "w3-text-theme");
				}
			}
			for (pre in article.elementsNamed("pre")) {
				pre.set("class", "w3-theme-light");
				pre.firstChild().set("class", "prettyprint");
			}
			for (p in article.elementsNamed("p")) {
				for (img in p.elementsNamed("img")) {
					img.set("class", "w3-image");
				}
			}
		}
		return xml.toString().replace("\t", "  ");
	}

	static function embed(embededTag : String, outerTag : String, content : String) : String {
		var contentParts = content.split("<" + embededTag + ">");
		contentParts[0] = "<" + outerTag + ">";
		for (i in 1...contentParts.length - 1) {
			contentParts[i] += "</" + outerTag + ">" + "<" + outerTag + ">";
		}
		contentParts[contentParts.length - 1] += "</" + outerTag + ">";
		return contentParts.join("<" + embededTag + ">");
	}

	static function highlight(html : String) : String {
		var attr = ~/class=([a-z]+)/g; //XHTML fix
		var alteredHtml = html;
		var openTagPos = alteredHtml.indexOf("<code>");
		while (openTagPos != -1) {
			var closeTagPos = alteredHtml.indexOf("</code>", openTagPos);
			var codeContent = alteredHtml.substring(openTagPos + 6, closeTagPos - 1);
			if (codeContent.indexOf("class") != -1) {
				codeContent = attr.replace(Highlighter.syntaxHighlightHaxe(codeContent), "class=\"$1\"");
				var contentBefore = alteredHtml.substring(0, openTagPos + 6);
				var contentAfter = alteredHtml.substring(closeTagPos, alteredHtml.length);
				alteredHtml = contentBefore + codeContent + contentAfter;
				openTagPos = openTagPos + codeContent.length + 7;
			} else {
				openTagPos = closeTagPos + 7;
			}
			openTagPos = alteredHtml.indexOf("<code>", openTagPos);
		}
		return alteredHtml;
	}

	static function header(title : String) : String {
		var titlelc = title.toLowerCase();
		return	'<!DOCTYPE html>\n' +
				'<html lang="en"><head>\n' +
				'	<title>BUW: Haxe / OpenFL GUI - $title</title>\n' +
				'	<meta charset="UTF-8"/>\n' +
				'	<link rel="stylesheet" href="https://www.w3schools.com/w3css/4/w3.css"/>\n' +
				'	<link rel="stylesheet" href="https://www.w3schools.com/lib/w3-theme-indigo.css">\n' +
				//~ '	<link rel="stylesheet" href="highlighter.min.css" />\n' +
				'	<style>' +
				'html pre,html pre code.prettyprint{font-family:consolas,monospace;white-space:pre;overflow-x:auto;font-size:14px;}' +
				'code.prettyprint{display:block;color:#333}' + //from hightlighter.min.css removed padding and margin
				'code.prettyprint .type{color:#0086b3;}' +
				'code.prettyprint .kwd{color:#00a;}' +
				'code.prettyprint .val{color:#44a;}' +
				'code.prettyprint .str,code.prettyprint .str .kwd,code.prettyprint .str .val,code.prettyprint .str .type{color:#a00;}' +
				'code.prettyprint .cmt{color:#998;font-style:italic;}' +
				'code.prettyprint .ereg,code.prettyprint .ereg .type{color:#f01;}' +
				'code.prettyprint .cmt .kwd,code.prettyprint .cmt .str,code.prettyprint .cmt .val,code.prettyprint .cmt .type{color:#998;}' +
				'	</style>\n' +
				'	<meta name="viewport" content="width=device-width, initial-scale=1">\n' +
				'	<meta name="description" content="BUW, Basic and Usefull Widgets is a GUI library for Haxe and OpenFL."/>' +
				'	<meta name="keywords" content="Widgets,GUI,Haxe,OpenFL,library"/>' +
				'	<meta name="author" content="Frank ENDRES (IT teacher)"/>' +
				'	<script>\n' +
				'function toggleMobileMenu() {\n' +
				'	var m = document.getElementById("mobileNav");\n' +
				'	if (m.className.indexOf("w3-show") == -1) {\n' +
				'		m.className += " w3-show";\n' +
				'	} else {\n' +
				'		m.className = m.className.replace(" w3-show", "");\n' +
				'	}\n' +
				'}\n' +
				'	</script>' +
				'</head><body>\n' +
				'	<header>\n' +
				'		<nav><div class="w3-top w3-theme-d5 w3-bar">\n' +
				'			<a id="home" href="index.html" class="w3-bar-item w3-button w3-hide-small">Home</a>\n' +
				'			<a target="_blank" href="demo/index.html" class="w3-bar-item w3-button w3-hide-small">Demo</a>\n' +
				'			<a id="news" href="news.html" class="w3-bar-item w3-button w3-hide-small">News</a>\n' +
				'			<a id="examples" href="examples.html" class="w3-bar-item w3-button w3-hide-small">Examples</a>\n' +
				'			<a id="uml" href="uml.html" class="w3-bar-item w3-button w3-hide-small">UML</a>\n' +
				'			<a target="_blank" href="api/buw/index.html" class="w3-bar-item w3-button w3-hide-small">API</a>\n' +
				'			<span class="w3-bar-item w3-hide-large w3-hide-medium">BUW: Haxe / OpenFL GUI</span>\n' +
				'			<a class="w3-bar-item w3-button w3-hover-black w3-right w3-hide-medium w3-hide-large" ' +
								'onclick="toggleMobileMenu(); return false;" title="Menu">&#9776;</a>\n' +
				'		</div>\n' +
				'		<div id="mobileNav" class="w3-theme-d5 w3-bar-block w3-hide w3-top">\n' +
				'			<a href="index.html" onclick="toggleMobileMenu()" class="w3-bar-item w3-button">Home</a>\n' +
				'			<a href="demo/index.html" onclick="toggleMobileMenu()" class="w3-bar-item w3-button">Demo</a>\n' +
				'			<a href="news.html" onclick="toggleMobileMenu()" class="w3-bar-item w3-button">News</a>\n' +
				'			<a href="examples.html" onclick="toggleMobileMenu()" class="w3-bar-item w3-button">Examples</a>\n' +
				'			<a href="uml.html" onclick="toggleMobileMenu()" class="w3-bar-item w3-button">UML</a>\n' +
				'			<a href="api/buw/index.html" onclick="toggleMobileMenu()" class="w3-bar-item w3-button">API</a>\n' +
				'		</div></nav>\n' +
				'		<h1 class="w3-center w3-text-theme w3-hide-small" style="margin-top : 1.5em">BUW: Haxe / OpenFL GUI - $title</h1>\n' +
				'		<br class="w3-hide-medium w3-hide-large" style="display : block; margin-top : 2.5em"/>' +
				'		<script>document.getElementById("$titlelc").className += " w3-theme-l1";</script>\n' +
				'	</header><section>';
	}

	static function footer() : String {
		return	'\n	</section>\n' +
				'	<footer class="w3-theme w3-container">frankendres@org.tuxfamily (inverted)</footer>\n' +
				'</body></html>';
	}
}
